from flask import Blueprint, request, jsonify
import geruecht.controller as gc
import geruecht.controller.ldapController as lc
import geruecht.controller.userController as uc
from datetime import datetime
from geruecht.model import BAR, MONEY
from geruecht.decorator import login_required

baruser = Blueprint("baruser", __name__)

ldap= lc.LDAPController(gc.ldapConfig['URL'], gc.ldapConfig['dn'])
userController = uc.UserController()


@baruser.route("/bar")
@login_required(groups=[BAR])
def _bar(**kwargs):
    """ Main function for Baruser

        Returns JSON-file with all Users, who hast amounts in this month.

        Returns:
            JSON-File with Users, who has amounts in this month
            or ERROR 401 Permission Denied
    """
    dic = {}
    users = userController.getAllUsersfromDB()
    for user in users:
        geruecht = None
        geruecht = user.getGeruecht(datetime.now().year)
        if geruecht is not None:
            month = geruecht.getMonth(datetime.now().month)
            amount = month[0] - month[1]
            all = geruecht.getSchulden()
            if all != 0:
                if all >= 0:
                    type = 'credit'
                else:
                    type = 'amount'
                dic[user.uid] = {"username": user.uid,
                                    "firstname": user.firstname,
                                    "lastname": user.lastname,
                                    "amount": abs(all),
                                    "locked": user.locked,
                                    "type": type
                                    }
    return jsonify(dic)


@baruser.route("/baradd", methods=['POST'])
@login_required(groups=[BAR])
def _baradd(**kwargs):
    """ Function for Baruser to add amount

        This function added to the user with the posted userID the posted amount.

        Returns:
            JSON-File with userID and the amount
            or ERROR 401 Permission Denied
    """
    data = request.get_json()
    userID = data['userId']
    amount = int(data['amount'])

    date = datetime.now()
    userController.addAmount(userID, amount, year=date.year, month=date.month)
    user = userController.getUser(userID)
    geruecht = user.getGeruecht(year=date.year)
    month = geruecht.getMonth(month=date.month)
    amount = abs(month[0] - month[1])
    all = geruecht.getSchulden()
    if all >= 0:
        type = 'credit'
    else:
        type = 'amount'
    dic = user.toJSON()
    dic['amount'] = abs(all)
    dic['type'] = type

    return jsonify(dic)


@baruser.route("/barGetUsers")
@login_required(groups=[BAR, MONEY])
def _getUsers(**kwargs):
    """ Get Users without amount

        This Function returns all Users, who hasn't an amount in this month.

        Returns:
            JSON-File with Users
            or ERROR 401 Permission Denied
    """
    retVal = {}
    retVal = ldap.getAllUser()
    return jsonify(retVal)

@baruser.route("/bar/storno", methods=['POST'])
@login_required(groups=[BAR])
def _storno(**kwargs):
    """ Function for Baruser to storno amount

            This function added to the user with the posted userID the posted amount.

            Returns:
                JSON-File with userID and the amount
                or ERROR 401 Permission Denied
        """
    data = request.get_json()
    userID = data['userId']
    amount = int(data['amount'])

    date = datetime.now()
    userController.addCredit(userID, amount, year=date.year, month=date.month)
    user = userController.getUser(userID)
    geruecht = user.getGeruecht(year=date.year)
    month = geruecht.getMonth(month=date.month)
    amount = abs(month[0] - month[1])
    all = geruecht.getSchulden()
    if all >= 0:
        type = 'credit'
    else:
        type = 'amount'
    dic = user.toJSON()
    dic['amount'] = abs(all)
    dic['type'] = type

    return jsonify(dic)

@baruser.route("/barGetUser", methods=['POST'])
@login_required(groups=[BAR])
def _getUser(**kwargs):
    data = request.get_json()
    username = data['userId']
    user = userController.getUser(username)
    amount = user.getGeruecht(datetime.now().year).getSchulden()
    if amount >= 0:
        type = 'credit'
    else:
        type = 'amount'

    retVal = user.toJSON()
    retVal['amount'] = amount
    retVal['type'] = type
    return jsonify(retVal)


@baruser.route("/search", methods=['POST'])
@login_required(groups=[BAR, MONEY])
def _search(**kwargs):
    data = request.get_json()
    searchString = data['searchString']
    retVal = ldap.searchUser(searchString)
    return jsonify(retVal)
